;**************************************************************
;*
;* Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 analoger Eingang
;*		1 
;*		2 
;*		3 
;*		4 
;*	PORTB:	0 LED	   
;*		1 LED	    
;*		2 LED	    
;*		3 LED	     
;*		4 LED	    
;*		5 LED	    
;*		6 LED	    
;*		7 LED	    
;*	PORTC:	0 
;*		1   
;*		2 PWM-Ausgang	       
;*		3      
;*		4 
;*		5  
;*		6 	    
;*		7 	    
;*
;**************************************************************
;
; sprut (zero) Bredendiek 04/2002
;
; ADC-Lernbeispiel:
;
; 16F876 wandelt die an RA0 anliegende Spannung in einen
; 10-Bit-Wert um
; die oberen 8 Bit des Wertes werden an PortB angezeigt
;
; 16F876 erzeugt an RC2 ein 2,5 kHz-Signal mit Tastverhltnis
; entsprechend der gemessenen Spannung (8-Bit Auflsung)
;
; Prozessortakt:  20 MHz
; Eingangsspannung: 0V ... 5V (Vss ... Vdd)
;
;**************************************************************
; Includedatei fr den 16F876 einbinden

	list p=16f876
	#include <P16f876.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

;**************************************************************
; Configuration festlegen:
; Power on Timer, kein Watchdog, HS-Oscillator, kein Brown out, kein LV-programming
	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF

;**********************************************************

; Variablen
wait	EQU	0x20

; Anfangsinitialisierung

init
; PWM vorbereiten
; Vorteiler 16:1 und Timer2 einschalten
	BSF	T2CON,T2CKPS1	; Vorteiler 16:1
	BSF	T2CON,TMR2ON	; Timer2 ein

; Frequenz auf 1,22 kHz einstellen
	BSF	STATUS,RP0	; Bank1
	MOVLW	D'255'		;
	MOVWF	PR2		; 1.22 kHz
	BCF	STATUS,RP0	; Bank1

; Tastverhltnis auf 50% einstellen
	MOVLW	D'127'		;
	MOVWF	CCPR1L		; 50% 

; RC2/CCP1 auf Ausgang stellen
	BSF	STATUS,RP0	; Bank1
	BCF	TRISC, 2	; RC2: output=0
	BCF	STATUS,RP0	; Bank 0

; PWM MODE mit CCP1 initialisieren
	CLRF	CCP1CON		; CCP1-Modus aus
	BSF	CCP1CON,CCP1M3	; CCP1-Modus PWM-Mode
	BSF	CCP1CON,CCP1M2	;

; ADC vorbereiten
; PortB vorbereiten
	bsf	STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'00000000'	; PortB alle output
	movwf	TRISB
	bcf	STATUS, RP0	; auf Bank 0 zurckschalten
	clrf	PORTB		; alle LEDs ausschalten

; ADC einschalten
	BSF	ADCON0, 0	; ADON=1

; ADC-Eingang AN0 auswhlen
	BCF	ADCON0, 5	; ADCHS2=0
	BCF	ADCON0, 4	; ADCHS1=0
	BCF	ADCON0, 3	; ADCHS0=0

; ADC speed fr 20 MHz einstellen
	BSF	ADCON0, 7	; ADCS1=1
	BCF	ADCON0, 6	; ADCS0=0

; Daten linksbndig
	BSF	STATUS,RP0	; Bank1
	BCF	ADCON1, 7	; ADFM=0
	BCF	STATUS,RP0	; Bank0


;**********************************************************
; Hauptprogrammschleife

Main
;Eingangsspannung wandeln
	BSF	ADCON0, 2	; ADC starten
loop
	BTFSC	ADCON0, 2	; ist der ADC fertig?
	GOTO	loop		; nein, weiter warten

; Wert nach PortB schreiben
	movfw	ADRESH		; obere 8 Bit auslesen
	movwf	PORTB		; obere 8 Bit nach PortB

; Wert an PWM bergeben
	MOVWF	CCPR1L		; obere 8 Bit sind PWM

; Warten, damit der ADC sich erholen kann
	clrf	wait
warten
	DECFSZ	wait, f
	goto	warten

	goto	Main

	end

;**********************************************************
